// --------------------------------------------
// HYPEDEMO.POV
// Copyright 1993, Truman Brown (CIS 71477,221)
// All Rights Reserved
// --------------------------------------------

// [ INCLUDES ]
#include "colors.inc"             // standard colors
#include "shapes.inc"             // standard shapes
#include "fovttb.inc"             // personal extended field of view values

// [ STUDIO ]
camera {
   location  < 0.000 90   -280 >  // dead center, up a bit, back a bit
   direction < 0.000  0 FoV_65 >
   up        < 0.000  1      0 >
   right     < 1.333  0      0 >
   look_at   < 0.000 68      0 >  // look down a bit
}
object { light_source { <  40 100 -100 > color White } }
object { light_source { < -50 150  -80 > color White } }
object { light_source { <-150 440 -195 > color White } }

// [ TEXTURES ]
#declare drop_texture1 =          // ripply semi-transparent water
   texture {
      color Cyan alpha 0.87
      ambient 0.2
      diffuse 0.8
      specular 1.2
      roughness 0.001
      refraction 1.0
      ior 1.33
      reflection 0.35
      ripples 0.9
      frequency 1.2
      scale <4 6 4>
      translate <0 93 0>
   }

#declare drop_texture2 =          // slightly reflective dimdark red
   texture {
   color Red
//      color CandyRed              // non-standard color: r=0.29 g=0.05 b=0.07
      ambient 0.1
      diffuse 0.35
      reflection 0.79
      specular 0.7
      roughness 0.001
      phong 1
      phong_size 150
      brilliance 0.15
   }

#declare rim_texture1 =           // metal ring texture
   texture {
      color Gold
      ambient 0.15
      diffuse 0.25
      reflection 0.15
      specular 0.9
      roughness 0.001
      brilliance 0.75
   }

/*
#declare wood_texture =           // image_mapped wood

   texture {                      // can be replaced with procedural wood texture
      image_map {
         0
         <1 0 -1>
         tga
         "ewalnut.tga"
         interpolate Bi
      }
      ambient 0.1
      diffuse 0.45
      specular 0.2
      roughness 0.001
      reflection 0.05
   }
*/

#declare floor_texture1 =
   texture {
      agate
      translate <-0.5 0 -0.5>
      scale <679 1 374>
      scale <0.6 0.6 0.6>
   }

#declare Gray25 = color red 0.25 green 0.25 blue 0.25

#declare wall_texture1 =          // flat bland gray with curvy holes
   texture {
      agate
      turbulence 0.7
      color_map {
         [0.00 0.40 color Gray25 alpha 0 color Gray25 alpha 0]
         [0.40 0.41 color Gray25 alpha 0 color Black  alpha 1]
         [0.41 0.59 color Black  alpha 1 color Black  alpha 1]
         [0.59 0.60 color Black  alpha 1 color Gray25 alpha 0]
         [0.60 1.01 color Gray25 alpha 0 color Gray25 alpha 0]
      }
      scale <200 235 40>
      translate <-50 35 100>
      scale <2.5 2.5 1>
      translate <-75 0 0>
      scale <0.75 0.75 1>
      ambient 0.15
      diffuse 0.35
      specular 0.8
      roughness 0.02
      reflection 0.25
      metallic
   }

// [ OBJECTS ]
#declare wall1 =                  // background
   object {
      box { <-99999 0 150> <99999 99999 185> }
      texture { wall_texture1 }
   }

#declare base =                   // base of drop shape
   object {                       // difference of a torus from a cylinder
      difference {                // radius at bottom = 22.5
         quadric {                // radius at top    = 10.0
            Cylinder_Y            // height           = 12.5
            scale <22.5 1 22.5>   // torus major radius should match radius of cylinder
         }                        // torus minor radius should match height of cylinder (see clipped_by below)
         //    Major radius: 22.5 Torus definition generated with
         //    Minor radius: 12.5 Truman's Torus Generator (TTG)
         //    Outer radius: 35.0 C:\TTG > TTG 10 35
         //    Inner radius: 10.0
         quartic { < 1 0 0 0 2 0 0 2 0 -1325 0 0 0 0 0 0 0 0 0 0 1 0 0 2 0 700 0 0 0 0 1 0 -1325 0 122500 > translate <0 12.5 0> }
      }
      clipped_by {
         plane { < 0 1 0 > 12.5 }
         plane { < 0 -1 0 > 0   }
      }
      bounded_by {
         box { <-22.5 0 -22.5> <22.5 12.5 22.5> }
      }
   }

#declare stem =                    // short stem before neck of drop shape
   object {                        // a clipped cylinder
      quadric {                    // radius = 10.0
         Cylinder_Y                // height =  1.5
         scale <10 1 10>
      }
      clipped_by {
         plane { < 0  1 0 > 1.5 }
         plane { < 0 -1 0 > 0 }
      }
      translate <0 12.5 0>         // move on top of base
   }

//                                 C:\HYPE > HYPE r-20 r20 a10 t45 ooutput.txt
//                                 make a clipped hyperboloid of one sheet
//                                 radius at both ends = 20
//                                 radius of neck = 10
//                                 tangent at clipped ends = 45
#declare neck =
   object {
      // Declaration created by Hype 1.0 (c) 1992 Truman Brown
      // 1st radius          = 20
      //     tangent angle   = 45
      //     complement      = 45
      // 2nd radius          = 20
      //     tangent angle   = 45
      //     complement      = 45
      // Angle of asymptotes = 40.893394649130904
      // Length              = 30
      // Unit length factor  =  0.033333333333333
      // A value             = 10
      // B value             =  8.660254037844386
      // C value             = 10
      // foci [+/-]          = 13.228756555322953
      quadric {
         <0.01 -0.013333333333333 0.01>
         <0.0   0.0               0.0 >
         <0.0   0.0               0.0 >
         -1
      }
      clipped_by {
         plane { < 0  1 0 > 15 }
         plane { < 0 -1 0 >  0 }   // instead of clipping at other end of hyperboloid
      }                            // clip at center
      bounded_by { box { <-20 0 -20> <20 15 20> } }
      translate <0 12.5 0>
      translate <0  1.5 0>         // move on top of stem
   }

//                                 C:\CM > CM x30 y30 r20 t45 c ooutput2.txt
//                                 create clipped sphere with a radius
//                                 at the clipped point of 20 - this
//                                 matches the radius of the end of the
//                                 hyperboloid.  The 45 degree angle also
//                                 matches the tangent of the hyperboloid.
//                                 The size of x and y is arbitrary, larger
//                                 values will make fatter drops
#declare bulb =
   object {
      // Declaration created by CircleMaster 1.0 (c) 1992 Truman Brown
      // a [radius on x-axis]            = 30.0
      // b [radius on y-axis]            = 33.541019662496907
      // c [radius on z-axis]            = 30.0
      // desired radius at intersection  = 20.0
      // y value of intersection point   = 25.0
      // desired tangent angle           = 45.0
      //    complement                   = 45.0
      // tangent angle achieved          = 45.0
      //    complement                   = 45.0
      // foci on Y-axis [+/-]            = 15.0
      // unit radius scaler              =  0.05
      // a after scaling                 =  1.5
      // b after scaling                 =  1.677050983124845
      // c after scaling                 =  1.5
      // y at intersection after scaling =  1.25
      // *** Don't forget to add a texture
      quadric {
         <0.001111111111111 0.00088888888888 0.001111111111111>
         <0.000000000000000 0.00000000000000 0.000000000000000>
         <0.000000000000000 0.00000000000000 0.000000000000000>
         -1.0
      }
      clipped_by { plane { <0 -1 0> 25 } }  // changed direction of clipping so that the bottom half disappears instead of the top
      translate <0 25 0>                    // move so that the clipped end is at the origin
      bounded_by { box { < -30 0 -30> <30 58.6 30> } }
      translate <0 12.5 0>
      translate <0  1.5 0>
      translate <0 15.0 0>                  // move on top of neck
   }

#declare disk =                             // just a simple disk
   object {
      plane { < 0 1 0 > 0 }
      clipped_by {
         quadric {
            Cylinder_Y
            scale <22.5 1 22.5>
            inverse
         }
         quadric {
            Cylinder_Y
            scale <30 1 30>
         }
      }
      bounded_by { box { <-30 -0.01 -30> <30 0.01 30> } }
   }

// --------------------------------------------------------------------------
// This file created by Truman's Torus Generator 1.1 (c) 1992 Truman Brown
// --------------------------------------------------------------------------
// Torus specs:
//    Major radius: 32.500000    C:\TTG > TTG 30 35
//    Minor radius: 2.500000
//    Outer radius: 35.000000
//    Inner radius: 30.000000
// ------------------------------------------------------------------------------

#declare rim =
   object {
      quartic { < 1 0 0 0 2 0 0 2 0 -2125 0 0 0 0 0 0 0 0 0 0 1 0 0 2 0 2100 0 0 0 0 1 0 -2125 0 1102500 > sturm }
      bounded_by { box { <-35 -2.5 -35> <35 2.5 35> } }
   }

#declare floor =                            // put a hole in the floor
   object {                                 // for the central droplet
      plane { < 0 1 0 > 0 }
      clipped_by {
         quadric {
            Cylinder_Y
            scale <35 1 35>
            inverse
         }
      }
   }

#declare drop =                             // snap the pieces together
   composite {                              // for the central droplet
      object { base texture { drop_texture1 } }
      object { stem texture { drop_texture1 } }
      object { neck texture { drop_texture1 } }
      object { bulb texture { drop_texture1 } }
      bounded_by { box { <-30 0 -30> <30 87.55 30> } }
   }

#declare dropbox =                          // pedestal
   object {
      box { <-30 0 -30> <30 10 30> }
   }

#declare babydrop =                         // snap the pieces together
   composite {                              // for the exploded droplets
      object { base translate <0 10 0> texture { drop_texture2 } }
      object { stem translate <0 14 0> texture { drop_texture2 } }
      object { neck translate <0 18 0> texture { drop_texture2 } }
      object { bulb translate <0 22 0> texture { drop_texture2 } }
      object { dropbox                 texture { drop_texture2 } }
      bounded_by { box { <-30 0 -30> <30 110.55 30> } }
      scale <0.65 1.25 0.65>
      translate <90 0 0>
   }

composite { drop  scale <1 1.45 1>           } // put the whole thing together
object    { disk  texture { drop_texture1  } } // and call it a day
object    { rim   texture { rim_texture1   } }
object    { floor texture { floor_texture1 } }
object    { wall1                            }

composite { babydrop rotate <0  45 0> }
composite { babydrop rotate <0 135 0> }
composite { babydrop rotate <0 225 0> }
composite { babydrop rotate <0 315 0> }
